precision highp float;

varying vec2 v_Tex; // uv texture coordinates

uniform vec2 effectTextureSize; // texture size (in pixels)
uniform sampler2D u_Tex; // texture
uniform float time; // time (in seconds?)

uniform float shapeX;        // X center      (in Pixels)
uniform float shapeY;        // Y center      (in Pixels)
uniform float shapeAngle;    // Rotation      (in Radians)
uniform float shapeWidth;    // Width         (in Pixels)
uniform float shapeHeight;   // Height        (in Pixels)
uniform float shapeFeather;  // Feather       (in Pixels)
uniform float showGuide;     // 0 or 1
uniform float blurStrength;  // 0 to 1

#define PI 3.141592653589793238
#define SQRT2 1.4142135623730951



float ellipse( vec2 ab, vec2 uv ){
    /*
    see here
    https://blog.chatfield.io/simple-method-for-distance-to-ellipse/
    */

    ( ab.x > ab.y ) ? ab = ab.yx, uv = abs(uv.yx) : uv = abs(uv);

    vec2 N1 = ab*uv, N2 = ab*ab;

    float U1 = N1.y/2.0;  float U5 = 4.0*U1;
    float U2 = N2.y-N2.x; float U6 = 6.0*U1;
    float U3 = N1.x-U2;   float U7 = 3.0*U3;
    float U4 = N1.x+U2;

    float t = 0.5;

    for( int i = 0; i < 3; i += 1 ){
        float F1 = t*(t*t*(U1*t+U3)+U4)-U1;
        float F2 = t*t*(U5*t+U7)+U4;
        float F3 = t*(U6*t+U7);
        t += (F1*F2)/(F1*F3-F2*F2);
    }

    t = clamp( t, 0.0, 1.0 );

    vec2 K = ab*vec2(1.0-t*t,2.0*t)/(t*t+1.0);
    float side = (uv.x - K.x < 0.0 ? -1.0 : 1.0);

    return distance( uv, K ) * side;
}

float sdfEllipse( vec2 P, vec2 O, float ang, float rad0, float rad1, float feather ){

    // rotate coordinates
    vec2 OP = P - O;
    float s = sin( ang );
    float c = cos( ang );
    vec2 OP_ = vec2( OP.x * c + OP.y * s, OP.x * s - OP.y * c );

    // get sdf
    float d = ellipse(vec2( rad0, rad1 ), OP_ );

    // feather
    d = smoothstep( 0.0, feather, d );

    return d;
}

void main( void ){

    vec2 UV = v_Tex;
    float screenWidth = effectTextureSize.x;
    float screenHeight = effectTextureSize.y;



    // base texture
    vec3 colTex1 = texture2D( u_Tex, UV ).rgb;

    // pixels
    float px = UV.x * screenWidth;
    float py = UV.y * screenHeight;



    //---------- get mask factor / blur value ----------

    // get sdf (or use 1-sdf to invert)
    float sdf = sdfEllipse( vec2(px,py), vec2(shapeX,shapeY), shapeAngle, shapeWidth, shapeHeight, shapeFeather + 1.0 );



    //---------- do blur ----------

    vec2 pixelSize = vec2( 1.0 / screenWidth, 1.0 / screenHeight );

    /*
    define blur kernel. (a strength table to multiply neighbouring colors by - must add up to 1)

    we are using Guassian ones, but skipping smallest values for speed
    see Pascal's triangle for the values.

    see here for how we should do this when multi-pass renders are implemented:
    https://www.rastergrid.com/blog/2010/09/efficient-gaussian-blur-with-linear-sampling/

    */

    float offset[15];
    offset[ 0] =-7.0;
    offset[ 1] =-6.0;
    offset[ 2] =-5.0;
    offset[ 3] =-4.0;
    offset[ 4] =-3.0;
    offset[ 5] =-2.0;
    offset[ 6] =-1.0;
    offset[ 7] = 0.0;
    offset[ 8] = 1.0;
    offset[ 9] = 2.0;
    offset[10] = 3.0;
    offset[11] = 4.0;
    offset[12] = 5.0;
    offset[13] = 6.0;
    offset[14] = 7.0;

    float kernel[15];
    /*
    // Guassian kernels
    const float PASCAL_SIZE = 16384.0;
    const float iWeight = 1.0 / PASCAL_SIZE;

    kernel[ 0] = 1.0    * iWeight;
    kernel[ 1] = 14.0   * iWeight;
    kernel[ 2] = 91.0   * iWeight;
    kernel[ 3] = 364.0  * iWeight;
    kernel[ 4] = 1001.0 * iWeight;
    kernel[ 5] = 2002.0 * iWeight;
    kernel[ 6] = 3003.0 * iWeight;
    kernel[ 7] = 3432.0 * iWeight;
    kernel[ 8] = 3003.0 * iWeight;
    kernel[ 9] = 2002.0 * iWeight;
    kernel[10] = 1001.0 * iWeight;
    kernel[11] = 364.0  * iWeight;
    kernel[12] = 91.0   * iWeight;
    kernel[13] = 14.0   * iWeight;
    kernel[14] = 1.0    * iWeight;
*/
    // Sine kernels
    kernel[0] = 0.002772;
    kernel[1] = 0.009788;
    kernel[2] = 0.024846;
    kernel[3] = 0.049614;
    kernel[4] = 0.081942;
    kernel[5] = 0.115302;
    kernel[6] = 0.140660;
    kernel[7] = 0.150151;
    kernel[8] = 0.140660;
    kernel[9] = 0.115302;
    kernel[10] = 0.081942;
    kernel[11] = 0.049614;
    kernel[12] = 0.024846;
    kernel[13] = 0.009788;
    kernel[14] = 0.002772;

    float offsetX;
    float offsetY;
    float weight;

    float expand = SQRT2 * 1.7 * sdf * blurStrength;

    vec3 colBlur = vec3(0.0);
    for ( int i = 0; i < 15; i += 1 ){ // skip some values
        for ( int j = 0; j < 15; j += 1 ){

            offsetX = offset[ i ] * expand;
            offsetY = offset[ j ] * expand;
            weight  = kernel[ i ] * kernel[ j ];

            colBlur += texture2D( u_Tex, UV + vec2( offsetX, offsetY ) * pixelSize  ).rgb * weight;
        }
    }

    // blend colors
    vec3 colOut = colBlur;



    //---------- onscreen guide lines ----------

    float kfeather = 0.4 / (shapeFeather + 1.0);
    colOut = mix( colOut, vec3(1.0), showGuide * 
        max(
            min(step(0.01,sdf),step(sdf,0.01 + kfeather)),
            min(step(0.99 - kfeather,sdf),step(sdf,0.99))
        ) * mix( 0.4, 1.0,
            max(
                step(mod( gl_FragCoord.x + gl_FragCoord.y, 10.0), 1.0 ),
                step(mod( gl_FragCoord.x - gl_FragCoord.y + 1.0, 10.0), 1.0 )
            )
        ) * 0.6
    );



    // out
    gl_FragColor = vec4( colOut, 1.0 );
}