precision highp float;

varying vec2 v_Tex;
uniform vec2 effectTextureSize;       // texture size (in pixels)
uniform sampler2D u_Tex;

uniform float radius;       // radius of blur (normalized)
uniform vec2 position;      // center of defocus mask (normalized)
uniform float feather;      // softness of defocus radius (normalized)
uniform vec2 direction;     // direction of blur (normalized)
uniform float intensity;    // intensity of blur

float cutCircle(vec2 uv, vec2 center, float radius, float tolerance) {
    vec2 dist = center - uv;
    return 1.0 - smoothstep(radius - tolerance,
    radius + tolerance,
    sqrt(dot(dist, dist)));
}

float cutCircle(float ratio, vec2 uv, vec2 center, float radius, float tolerance) {
    uv.x *= ratio;
    center.x *= ratio;
    vec2 dist = center - uv;
    return 1.0 - smoothstep(radius - tolerance,
    radius + tolerance,
    sqrt(dot(dist, dist)));
}

const float pi = radians(180.);
int samples;
float sigma;
float sigmaSqrX2;
float sigmaRoot2Pi;

// see https://en.wikipedia.org/wiki/Gaussian_blur for the formulae of a Gaussian function
// here the top (dividend) and bottom (divisor) lines are calculated separetly for clarity
float gaussian(float i) {
    float top = exp(-i * i / sigmaSqrX2);
    float bot = sigmaRoot2Pi;
    return top / bot;
}

// since the blur is symmetrical
// we can optimise, reducing the loop by half
vec4 blurOptimised(sampler2D sp, vec2 uv, vec2 scale, vec2 direction) {
    vec2 offset = vec2(0, 0);
    float weight = 0.;
    vec4 col;
    float accum;

    for (int i = -samples / 2; i <= samples / 2; ++i) {
        offset = float(i) * direction;
        weight = gaussian(float(i));
        col += texture2D(sp, uv + scale * offset) * weight;
        accum += weight;
    }

    return col / accum;
}

void main(void) {

    vec2 position = position;
    float intensity = mix(0.25, 1.0, intensity);
    float radius = mix(0.1, 0.35, radius);
    float feather = mix(0.025, 0.125, feather);

    float ratio = effectTextureSize.x / effectTextureSize.y;
    float circle = 1.0 - cutCircle(ratio, v_Tex, position, radius, feather);

    if (circle == 0.0 || length(direction) == 0.0) {
        gl_FragColor = texture2D(u_Tex, v_Tex);
    } else {
        samples = int(50.0 * circle * intensity) + 1;
        sigma = float(samples) * 0.25;
        sigmaSqrX2 = 2. * sigma * sigma;
        sigmaRoot2Pi = sigma * sqrt(2. * pi);

        vec2 ps = 1.0 / effectTextureSize;
        vec4 newColor = blurOptimised(u_Tex, v_Tex, ps, normalize(direction));

        gl_FragColor = newColor;
    }

}